import os
import sys
import json
import pathlib
from dotenv import load_dotenv
from eth_utils.address import to_checksum_address
from web3 import Web3
#from web3.middleware.geth_poa import geth_poa_middleware
from web3.middleware import ExtraDataToPOAMiddleware
from eth_account import Account

current_dir = os.getcwd()
dotenv_path = os.path.join(current_dir, ".env")
# Load environment variables from .env file
load_dotenv(dotenv_path=dotenv_path)

# Environment variables
PRIVATE_KEY = os.getenv("PRIVATE_KEY", "")
BLOCKCHAIN_NETWORK = os.getenv("BLOCKCHAIN_NETWORK", "Bloxberg_Testnet")

# Default values
NETWORK_RPC = "https://core.bloxberg.org"
IMAGE_REGISTRY_ADDRESS = (
    "0x15D73a742529C3fb11f3FA32EF7f0CC3870ACA31"  # Bloxberg testnet
)
CHAIN_ID = 8995
GAS = 9000000
GAS_PRICE = 1


def set_vars(network=""):
    global NETWORK_RPC, IMAGE_REGISTRY_ADDRESS, CHAIN_ID, GAS, GAS_PRICE, BLOCKCHAIN_NETWORK
    if network:
        BLOCKCHAIN_NETWORK = network
    if "Bloxberg" in BLOCKCHAIN_NETWORK:
        IMAGE_REGISTRY_ADDRESS = "0x15D73a742529C3fb11f3FA32EF7f0CC3870ACA31"
        NETWORK_RPC = "https://core.bloxberg.org"
        CHAIN_ID = 8995
        GAS = 9000000
        GAS_PRICE = Web3.to_wei(1, "mwei")  # 1 Mwei
    elif "Polygon" in BLOCKCHAIN_NETWORK:
        if "Mainnet" in BLOCKCHAIN_NETWORK:
            NETWORK_RPC = "https://polygon-rpc.com"
            IMAGE_REGISTRY_ADDRESS = "0x689f3806874d3c8A973f419a4eB24e6fBA7E830F"
            CHAIN_ID = 137
            GAS = 20000000
            GAS_PRICE = Web3.to_wei(40500500010, "wei")
        else:
            NETWORK_RPC = "https://rpc-amoy.polygon.technology"
            IMAGE_REGISTRY_ADDRESS = "0xF7F4eEb3d9a64387F4AcEb6d521b948E6E2fB049"
            CHAIN_ID = 80001
            GAS = 20000000
            GAS_PRICE = Web3.to_wei(1300000010, "wei")
    else:
        # Default to Bloxberg Testnet if no matching network
        IMAGE_REGISTRY_ADDRESS = "0x15D73a742529C3fb11f3FA32EF7f0CC3870ACA31"
        NETWORK_RPC = "https://core.bloxberg.org"
        CHAIN_ID = 8995
        GAS = 9000000
        GAS_PRICE = Web3.to_wei(1, "mwei")  # 1 Mwei


# Set variables based on the current network
set_vars()


def is_string_private_key(private_key):
    try:
        key = private_key
        if not key.startswith("0x"):
            key = f"0x{private_key}"
        Account().from_key(key)
        return "OK"
    except Exception as e:
        return str(e)


def check_account_balance():
    try:
        w3 = Web3(Web3.HTTPProvider(NETWORK_RPC))
        if PRIVATE_KEY:
            account = Account().from_key(PRIVATE_KEY)
        else:
            print("Error: PRIVATE_KEY is not set.")
            return 0
        balance = w3.eth.get_balance(account.address)
        return Web3.from_wei(balance, "ether")
    except Exception as e:
        print(e)
        return 0


class ImageRegistry:
    def __init__(self):
        try:
            self.image_registry_abi = self.read_contract_abi("image_registry.abi")
            self.image_registry_address = IMAGE_REGISTRY_ADDRESS
            print(f"imageRegistryAddress: {self.image_registry_address}")
            # self.provider = Web3(Web3.HTTPProvider(NETWORK_RPC))
            # print(f"Network RPC: {NETWORK_RPC}")
            self.provider = self.newProvider(NETWORK_RPC)

            # # Inject middleware if needed
            # if "Bloxberg" in BLOCKCHAIN_NETWORK or "Polygon" in BLOCKCHAIN_NETWORK:
            #     self.provider.middleware_onion.inject(geth_poa_middleware, layer=0)

            if PRIVATE_KEY:
                # print("Using private key from environment variable")
                self.acct = Account().from_key(PRIVATE_KEY)
                self.provider.eth.default_account = self.acct.address
            else:
                _private_key = Account().create().key
                self.acct = Account().from_key(_private_key)
                self.provider.eth.default_account = self.acct.address

            self.image_registry_contract = self.provider.eth.contract(
                address=to_checksum_address(self.image_registry_address),
                abi=self.image_registry_abi,
            )
        except Exception as e:
            print(e)

    def newProvider(self, url: str) -> Web3:
        _w3 = Web3(Web3.HTTPProvider(url))
        #_w3.enable_unstable_package_management_api()
        _w3.middleware_onion.inject(ExtraDataToPOAMiddleware, layer=0)
        return _w3

    def read_contract_abi(self, contract_name):
        file_path = pathlib.Path(__file__).parent / contract_name
        with open(file_path, "r") as f:
            return json.load(f)

    def add_trusted_zone_cert(
        self,
        cert_content,
        ipfs_hash,
        image_name,
        docker_compose_hash,
        enclave_name_trustedzone,
        fee,
    ):
        print("Adding trusted zone cert to image registry")
        try:
            nonce = self.provider.eth.get_transaction_count(self.acct.address)
            gas_price = GAS_PRICE if GAS_PRICE != 1 else self.provider.to_wei(1, "mwei")
            txn = self.image_registry_contract.functions.addTrustedZoneImage(
                ipfs_hash,
                cert_content,
                "v3",
                image_name,
                docker_compose_hash,
                enclave_name_trustedzone,
                int(fee),
            ).build_transaction(
                {
                    "nonce": nonce,
                    "gas": GAS,
                    "gasPrice": gas_price,
                    "chainId": CHAIN_ID,
                    "from": self.acct.address,
                }
            )

            signed_txn = self.provider.eth.account.sign_transaction(
                txn, private_key=PRIVATE_KEY
            )
            tx_hash = self.provider.eth.send_raw_transaction(signed_txn.raw_transaction)
            print(f"Transaction sent: {tx_hash.hex()}")

            receipt = self.provider.eth.wait_for_transaction_receipt(tx_hash)
            if receipt.status == 1:
                print("Adding trusted zone cert transaction was successful!")
            else:
                print("Adding trusted zone cert transaction was UNSUCCESSFUL!")
                sys.exit(1)
        except Exception as e:
            print(f"An error occurred while sending transaction: {e}")

    def add_secure_lock_image_cert(
        self,
        cert_content,
        ipfs_hash,
        image_name,
        version,
        docker_compose_hash,
        enclave_name_securelock,
        fee,
    ):
        try:
            print("Adding secure lock image cert to image registry")

            if "Polygon" in BLOCKCHAIN_NETWORK:
                print("Polygon network")
                nonce = self.provider.eth.get_transaction_count(
                    self.acct.address, "pending"
                )
                gas_price = self.provider.eth.gas_price
                gas_price = int(gas_price * 1.1)  # Increase gas price by 10%
            else:
                nonce = self.provider.eth.get_transaction_count(self.acct.address)
                gas_price = (
                    GAS_PRICE if GAS_PRICE != 1 else self.provider.to_wei(1, "mwei")
                )

            txn = self.image_registry_contract.functions.addImage(
                ipfs_hash,
                cert_content,
                version,
                image_name,
                docker_compose_hash,
                enclave_name_securelock,
                int(fee),
            ).build_transaction(
                {
                    "nonce": nonce,
                    "gas": GAS,
                    "gasPrice": gas_price,
                    "chainId": CHAIN_ID,
                    "from": self.acct.address,
                }
            )

            signed_txn = self.provider.eth.account.sign_transaction(
                txn, private_key=PRIVATE_KEY
            )
            tx_hash = self.provider.eth.send_raw_transaction(signed_txn.raw_transaction)
            print(f"Transaction sent: {tx_hash.hex()}")

            receipt = self.provider.eth.wait_for_transaction_receipt(tx_hash)
            if receipt.status == 1:
                print("Adding secure lock image cert transaction was successful!")
            else:
                print("Image certificates already exist for this image!")
        except Exception as e:
            # Uncomment the next line to see the actual error
            print(f"Error: {str(e)}")
            print("Image certificates already exist for this image!")

    def get_image_public_key_cert(self, ipfs_hash):
        try:
            print("Getting image cert from image registry")
            cert = self.image_registry_contract.functions.getImageCertPublicKey(
                ipfs_hash
            ).call()
            print("Image Public Key Certificate:", cert)
            return cert
        except Exception as e:
            print(f"Error retrieving image public key certificate: {str(e)}")
            return None

    def get_image_details(self, ipfs_hash):
        try:
            details = self.image_registry_contract.functions.imageDetails(
                ipfs_hash
            ).call()
            return details
        except Exception as e:
            # Uncomment to see the actual error
            print(f"Error: {str(e)}")
            return ("", "", "")

    def _get_latest_image_version_public_key(self, project_name, version):
        try:
            public_key_tuple = (
                self.image_registry_contract.functions.getLatestImageVersionPublicKey(
                    project_name, version
                ).call()
            )
            # The function returns a tuple, extract the fields as needed
            return public_key_tuple
        except Exception as e:
            # Uncomment to see the actual error
            # print(f"Error: {str(e)}")
            return ("", "", "")


def main(
    network_name="", project_name="", version_arg="", private_key_arg="", action=""
):
    global current_dir
    current_dir = os.getcwd()
    dotenv_path = os.path.join(current_dir, ".env")
    # Load environment variables from .env file
    load_dotenv(dotenv_path=dotenv_path)
    global PRIVATE_KEY, BLOCKCHAIN_NETWORK, NETWORK_RPC, IMAGE_REGISTRY_ADDRESS, CHAIN_ID, GAS, GAS_PRICE
    # Environment variables
    PRIVATE_KEY = os.getenv("PRIVATE_KEY", "")
    BLOCKCHAIN_NETWORK = os.getenv("BLOCKCHAIN_NETWORK", "Bloxberg_Testnet")

    # Default values
    NETWORK_RPC = "https://core.bloxberg.org"
    IMAGE_REGISTRY_ADDRESS = (
        "0x15D73a742529C3fb11f3FA32EF7f0CC3870ACA31"  # Bloxberg testnet
    )
    CHAIN_ID = 8995
    GAS = 9000000
    GAS_PRICE = 1

    if action == "validateAddress":
        res = "OK"
        if private_key_arg:
            res = is_string_private_key(private_key_arg)
        print(res)
        # sys.exit(0)
        return res

    if network_name and project_name:
        set_vars(network_name)
    else:
        set_vars()

    if action == "checkBalance":
        balance = check_account_balance()
        print(f"{balance} gas")
        return str(balance)
        # sys.exit(0)

    if action == "getTrustedZoneCert":
        image_registry = ImageRegistry()
        public = image_registry._get_latest_image_version_public_key(
            project_name, version_arg
        )[1]
        print(public)
        return public
        # sys.exit(0)

    image_registry = ImageRegistry()

    if action == "registerSecureLockImage":
        secure_lock = ""
        with open("./registry/certificate.securelock.crt", "r") as f:
            secure_lock = f.read()
        ipfs_hash = os.getenv("IPFS_HASH", "")
        print(f"ipfsHash: {ipfs_hash}")
        ipfs_docker_compose_hash = os.getenv("IPFS_DOCKER_COMPOSE_HASH", "")
        print(f"ipfsDockerComposeHash: {ipfs_docker_compose_hash}")
        image_name = os.getenv("PROJECT_NAME", "")
        print(f"imageName: {image_name}")
        version = os.getenv("VERSION", "")
        enclave_name_securelock = os.getenv("ENCLAVE_NAME_SECURELOCK", "")
        print(f"enclaveNameSecureLock: {enclave_name_securelock}")
        fee = os.getenv("DEVELOPER_FEE", "0")
        print(f"fee: {fee}")
        image_registry.add_secure_lock_image_cert(
            secure_lock,
            ipfs_hash,
            image_name,
            version,
            ipfs_docker_compose_hash,
            enclave_name_securelock,
            fee,
        )
        # sys.exit(0)
        return "OK"
    if action == "getImagePublicKey":
        ipfs_hash = os.getenv("IPFS_HASH", "")
        image_registry.get_image_public_key_cert(ipfs_hash)
        # sys.exit(0)
        return

    print(f"Checking image: '{project_name}' on the {BLOCKCHAIN_NETWORK} blockchain...")
    # print(f"Arguments: {args}")
    try:
        image_hash = image_registry._get_latest_image_version_public_key(
            project_name, version_arg if version_arg else "v3"
        )[0]
    except Exception as e:
        image_hash = ""
        # print(f"Error: {str(e)}")

    # print(f"Image hash: {image_hash}")
    if not image_hash:
        print(
            f"Image: '{project_name}' is available on the {BLOCKCHAIN_NETWORK} blockchain."
        )
        return "OK"
    print(f"Image hash: {image_hash}")
    image_owner = image_registry.get_image_details(image_hash)[0]
    if private_key_arg:
        if is_string_private_key(private_key_arg) == "OK":
            account = Account().from_key(private_key_arg)
            if image_owner.lower() != account.address.lower():
                print(
                    f"!!! Image: '{project_name}' is owned by '{image_owner}'.\nYou are not the account holder of the image.\nPlease change the project name and try again.\n"
                )
                return "You are not the account holder of the image."
    if image_owner and not private_key_arg:
        print(
            f"Image: '{project_name}' is owned by '{image_owner}'.\nIf you are not the account holder, you will not be able to publish your project with the current name. Please change the project name and try again.\n"
        )
        # sys.exit(0)
        return "You are not the account holder of the image."

    print(
        f"Image: '{project_name}' is available on the {BLOCKCHAIN_NETWORK} blockchain."
    )


if __name__ == "__main__":
    try:
        args = sys.argv[1:]
        if len(args) >= 5:
            network_name, project_name, version_arg, private_key_arg, action = args[:5]
        elif len(args) == 4:
            network_name, project_name, version_arg, private_key_arg = args[:4]
            action = ""
        elif len(args) == 3:
            network_name, project_name, version_arg = args[:3]
            action = ""
            private_key_arg = ""
        elif len(args) == 2:
            network_name, project_name = args[:2]
            action = ""
            version_arg = ""
            private_key_arg = ""
        elif len(args) == 1:
            action = ""
            network_name = args[0]
            project_name = ""
            version_arg = ""
            private_key_arg = ""

        #     network_name = ""
        #     project_name = ""
        #     version_arg = ""
        #     private_key_arg = ""
        #     action = ""
        main(network_name, project_name, version_arg, private_key_arg, action)
    except Exception as e:
        # Uncomment to see the actual error
        print(f"Error: {str(e)}")
        # sys.exit(0)



import os
import sys
import subprocess
import re
import json
import shutil
import requests
from pathlib import Path
from dotenv import load_dotenv
import os
from datetime import datetime, timedelta
from cryptography import x509
from cryptography.x509.oid import NameOID, ExtensionOID
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.hazmat.primitives.asymmetric import rsa
import time


def write_env(key, value):
    env_file = os.path.join(current_dir, ".env")
    if not os.path.exists(env_file):
        with open(env_file, "w") as f:
            f.write(f"{key}={value}\n")
        return

    updated = False
    with open(env_file, "r") as f:
        lines = f.readlines()

    with open(env_file, "w") as f:
        for line in lines:
            if line.startswith(f"{key}="):
                f.write(f"{key}={value}\n")
                updated = True
            else:
                f.write(line)
        if not updated:
            f.write(f"{key}={value}\n")


def prompt(question, default_value=None):
    """
    Prompt user for input with an optional default value.
    """
    if default_value:
        question = f"{question} (default value: {default_value}) "
    else:
        question = f"{question} "
    user_input = input(question).strip()
    if not user_input and default_value is not None:
        return default_value
    return user_input


def prompt_options(message, options, default_option):
    while True:
        answer = input(message).strip().lower()
        if not answer:
            print(f"No option selected. Defaulting to {default_option}.")
            return default_option
        elif answer in options:
            return answer
        else:
            print(
                f'Invalid option "{answer}". Please enter one of: {", ".join(options)}.'
            )


def run_docker_command(service):
    command = f"docker-compose run -e SCONE_LOG=INFO -e SCONE_HASH=1 {service}"
    try:
        output = (
            subprocess.check_output(
                command, shell=True, cwd=run_dir, stderr=subprocess.STDOUT
            )
            .decode()
            .strip()
        )

        print(f"Output of {command}: {output}")
        # Filter the output
        filtered_output = "\n".join(
            line
            for line in output.split("\n")
            if not re.search(
                r"Creating|Created|Recreated|Recreate|Pulling|latest|Digest|Starting|Started|msg=", line
            )
        )
        return filtered_output
    except subprocess.CalledProcessError as e:
        print(f"Error running command '{command}': {e.output.decode()}")
        return ""


def process_yaml_template(template_file, output_file, replacements):
    if not os.path.exists(template_file):
        print(f"Error: Template file {template_file} not found!")
        sys.exit(1)
    with open(template_file, "r") as f:
        content = f.read()
    for key, value in replacements.items():
        content = content.replace(f"__{key}__", value)
    with open(output_file, "w") as f:
        f.write(content)
    # Check for remaining placeholders
    remaining_placeholders = re.findall(r"__.*?__", content)
    if remaining_placeholders:
        print("Remaining placeholders:", ", ".join(remaining_placeholders))
    else:
        print("No placeholders found.")


def main():
    load_dotenv()
    if not os.path.exists(".env"):
        print("Error: .env file not found")
        sys.exit(1)

    global current_dir, run_dir
    current_dir = os.getcwd()
    print(f"currentDir: {current_dir}")
    run_dir = Path(__file__).resolve().parent / "run"
    os.chdir(run_dir)
    # print("run_dir: ", run_dir)
    registry_path = os.path.join(current_dir, "registry")
    os.environ["REGISTRY_PATH"] = registry_path

    templateName = os.getenv("TRUSTED_ZONE_IMAGE", "etny-pynithy-testnet")
    isMainnet = False if "testnet" in templateName.lower() else True

    # Backup and restore docker-compose templates
    backup_files = ["docker-compose.yml.tmpl", "docker-compose-final.yml.tmpl"]
    for file in backup_files:
        if not os.path.exists(file):
            print(f"Error: {file} not found!")
            continue
        shutil.copyfile(file, file.replace(".tmpl", ""))
    print()
    # Run docker commands to get MRENCLAVE values
    mrenclave_securelock = run_docker_command("etny-securelock")
    print(f"MRENCLAVE_SECURELOCK: {mrenclave_securelock}")

    write_env("MRENCLAVE_SECURELOCK", mrenclave_securelock)

    # Process YAML template for etny-securelock
    ENCLAVE_NAME_SECURELOCK = os.getenv("ENCLAVE_NAME_SECURELOCK", "")
    print(f"\nENCLAVE_NAME_SECURELOCK: {ENCLAVE_NAME_SECURELOCK}")

    envPredecessor = os.getenv("PREDECESSOR_HASH_SECURELOCK", "EMPTY")
    PREDECESSOR_HASH_SECURELOCK = "EMPTY"
    PREDECESSOR_PROJECT_NAME = "EMPTY"
    PREDECESSOR_VERSION = "EMPTY"
    if envPredecessor != "EMPTY":
        PREDECESSOR_HASH_SECURELOCK = envPredecessor.split("$$$%$")[0]
        PREDECESSOR_PROJECT_NAME = envPredecessor.split("$$$%$")[1]
        PREDECESSOR_VERSION = envPredecessor.split("$$$%$")[2]

    print(f"PREDECESSOR_HASH_SECURELOCK: {PREDECESSOR_HASH_SECURELOCK}")
    print(f"PREDECESSOR_PROJECT_NAME: {PREDECESSOR_PROJECT_NAME}")
    print(f"PREDECESSOR_VERSION: {PREDECESSOR_VERSION}")

    if (
        PREDECESSOR_HASH_SECURELOCK != "EMPTY"
        and PREDECESSOR_PROJECT_NAME != os.getenv("PROJECT_NAME")
        and PREDECESSOR_VERSION != os.getenv("VERSION")
    ):
        PREDECESSOR_HASH_SECURELOCK = "EMPTY"

    replacements_securelock = {
        "PREDECESSOR": (
            f"# predecessor: {PREDECESSOR_HASH_SECURELOCK}"
            if PREDECESSOR_HASH_SECURELOCK == "EMPTY"
            else f"predecessor: {PREDECESSOR_HASH_SECURELOCK}"
        ),
        "MRENCLAVE": mrenclave_securelock,
        "ENCLAVE_NAME": ENCLAVE_NAME_SECURELOCK,
    }

    process_yaml_template(
        "etny-securelock-test.yaml.tpl",
        "etny-securelock-test.yaml",
        replacements_securelock,
    )
    print()
    # Generate certificates if needed
    key_pem_path = "key.pem"
    cert_pem_path = "cert.pem"
    if (
        PREDECESSOR_HASH_SECURELOCK != "EMPTY"
        and os.path.exists(key_pem_path)
        and os.path.exists(cert_pem_path)
    ):
        print("Skipping key pair generation and certificate creation.")
        print("Using existing key.pem and cert.pem files.")
    else:
        print("# Generating cert.pem and key.pem files")

        # Generate a key pair
        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=4096,
        )

        # Get the public key
        public_key = private_key.public_key()

        # Get ENCLAVE_NAME_SECURELOCK from environment variable or default value
        organization_name = os.getenv(
            "ENCLAVE_NAME_SECURELOCK", "Internet Widgits Pty Ltd"
        )

        # Build subject and issuer names (self-signed certificate)
        subject = issuer = x509.Name(
            [
                x509.NameAttribute(NameOID.COUNTRY_NAME, "AU"),
                x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, "Some-State"),
                x509.NameAttribute(NameOID.ORGANIZATION_NAME, organization_name),
            ]
        )

        # Set validity period (not before one year ago, not after two years from now)
        valid_from = datetime.utcnow() - timedelta(days=365)
        valid_to = valid_from + timedelta(days=3 * 365)  # Valid for 3 years total

        # Serial number (use 1 for consistency)
        serial_number = 1

        # Build the certificate
        builder = x509.CertificateBuilder()
        builder = builder.subject_name(subject)
        builder = builder.issuer_name(issuer)
        builder = builder.public_key(public_key)
        builder = builder.serial_number(serial_number)
        builder = builder.not_valid_before(valid_from)
        builder = builder.not_valid_after(valid_to)

        # Add extensions
        # 1. Subject Key Identifier
        builder = builder.add_extension(
            x509.SubjectKeyIdentifier.from_public_key(public_key), critical=False
        )

        # 2. Authority Key Identifier
        builder = builder.add_extension(
            x509.AuthorityKeyIdentifier.from_issuer_public_key(public_key),
            critical=False,
        )

        # 3. Basic Constraints (mark as CA)
        builder = builder.add_extension(
            x509.BasicConstraints(ca=True, path_length=None), critical=True
        )

        # Self-sign the certificate
        certificate = builder.sign(
            private_key=private_key,
            algorithm=hashes.SHA256(),
        )

        # Serialize private key to PEM format (PKCS8)
        private_key_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )

        # Serialize certificate to PEM format
        certificate_pem = certificate.public_bytes(
            encoding=serialization.Encoding.PEM,
        )

        # Write private key and certificate to files
        with open("key.pem", "wb") as f:
            f.write(private_key_pem)

        with open("cert.pem", "wb") as f:
            f.write(certificate_pem)

        print("# Generated cert.pem and key.pem files")

    # Read certificates and data
    with open(cert_pem_path, "rb") as f:
        cert_data = f.read()
    with open(key_pem_path, "rb") as f:
        key_data = f.read()
    with open("etny-securelock-test.yaml", "rb") as f:
        yaml_data = f.read()

    # Set up the request headers
    headers = {"Content-Type": "application/octet-stream"}
    print()
    # Perform the HTTPS POST request
    try:
        # Create a session to manage certificates and SSL settings
        session = requests.Session()
        session.verify = False  # Equivalent to rejectUnauthorized: false
        session.cert = ("cert.pem", "key.pem")  # Provide the client cert and key

        # Perform the POST request
        response = session.post(
            "https://scone-cas.cf:8081/session", data=yaml_data, headers=headers
        )
        print(f"{response.json()}")
        # response.raise_for_status()  # Raise an exception for HTTP errors
        # print(f"Response status code: {response.status_code}")
        # print(f"Response text: {response.text}")
        # Write the response data to 'predecessor.json'
        with open("predecessor.json", "w", encoding="utf-8") as f:
            json.dump(response.json(), f, indent=2)
        print("# Updated session file for securelock and saved to predecessor.json")

        response_data = response.json()
        pred = response_data.get("hash", "EMPTY")
        project_name = os.getenv("PROJECT_NAME")
        version = os.getenv("VERSION")

        if pred != "EMPTY":
            predecessor_hash_securelock = (
                f"{pred}$$$%${project_name}$$$%${version}" or "EMPTY"
            )
            write_env("PREDECESSOR_HASH_SECURELOCK", predecessor_hash_securelock)
            os.environ["PREDECESSOR_HASH_SECURELOCK"] = predecessor_hash_securelock
        else:
            predecessor_hash_securelock = "EMPTY"
            write_env("PREDECESSOR_HASH_SECURELOCK", predecessor_hash_securelock)
            os.environ["PREDECESSOR_HASH_SECURELOCK"] = predecessor_hash_securelock

        if predecessor_hash_securelock == "EMPTY":
            print("Error: Could not update session file for securelock")
            print(
                "Please change the name/version of your project (using ecld-init or by editing .env file) and run the scripts again. Exiting."
            )
            sys.exit(1)

        print()
        print("Scone CAS registration successful.")
        print()

    except requests.RequestException as error:
        print("Scone CAS error:", error)
        print("Error: Could not update session file for securelock")
        print(
            "Please change the name/version of your project (using ecld-init or by editing .env file) and run the scripts again. Exiting."
        )
        sys.exit(1)

    ENCLAVE_NAME_TRUSTEDZONE = "etny-pynithy-trustedzone-v3-testnet-0.1.12"
    if isMainnet:
        ENCLAVE_NAME_TRUSTEDZONE = "ecld-pynithy-trustedzone-v3-3.0.0"
    print()
    # Update docker-compose files
    print("# Update docker-compose files")
    files = ["docker-compose.yml", "docker-compose-final.yml"]
    for file in files:
        if not os.path.exists(file):
            print(f"Error: {file} not found!")
            continue
        print(f"Processing {file}")
        with open(file, "r") as f:
            content = f.read()
        content = content.replace(
            "__ENCLAVE_NAME_SECURELOCK__", ENCLAVE_NAME_SECURELOCK
        ).replace("__ENCLAVE_NAME_TRUSTEDZONE__", ENCLAVE_NAME_TRUSTEDZONE)
        with open(file, "w") as f:
            f.write(content)
        remaining_placeholders = re.findall(r"__.*?__", content)
        if remaining_placeholders:
            print(
                f"Placeholders still found in {file}: {', '.join(remaining_placeholders)}"
            )
        else:
            print(f"Ok, No placeholders found in {file}")

    print()

    if os.path.exists("certificate.securelock.crt"):
        os.remove("certificate.securelock.crt")

    print("Fetching PUBLIC_KEY_SECURELOCK")
    PUBLIC_KEY_SECURELOCK_RES = ""
    try:
        output = (
            subprocess.check_output(
                "docker-compose run etny-securelock",
                shell=True,
                cwd=run_dir,
                stderr=subprocess.STDOUT,
            )
            .decode()
            .strip()
        )
        print("Output of docker-compose run etny-securelock:")
        lines = output.split("\n")
        publicKeyLine = next((line for line in lines if "PUBLIC_KEY:" in line), None)
        PUBLIC_KEY_SECURELOCK_RES = (
            publicKeyLine.replace(".*PUBLIC_KEY:\s*", "").strip()
            if publicKeyLine
            else ""
        )
    except subprocess.CalledProcessError as e:
        print(f"Could not fetch PUBLIC_KEY_SECURELOCK {e}")
        PUBLIC_KEY_SECURELOCK_RES = ""

    if not PUBLIC_KEY_SECURELOCK_RES:
        print("\n\nFor publishing the eclave, the public key needs to be extracted and for this SGX technology is required.\nIt seems that your machine is not SGX compatible.\n")
        should_generate_certificates = prompt(
            "Do you want to use Ethernity Cloud public public key extraction service? (yes/no):",
            default_value="yes",
        ).lower()
        print()
        if should_generate_certificates != "yes":
            print("Exiting.")
            sys.exit(1)
        print()
        print(
            "\nExtracting certificate using the Ethernity Cloud certificate extraction service...\n"
        )
        print("**** Started ipfs initial pining ****")
        if os.path.exists("IPFS_HASH.ipfs"):
            os.remove("IPFS_HASH.ipfs")
        if os.path.exists("IPFS_DOCKER_COMPOSE_HASH.ipfs"):
            os.remove("IPFS_DOCKER_COMPOSE_HASH.ipfs")
        print("Upload docker-compose-final.yml to IPFS")
        import ethernity_cloud_sdk_py.commands.pynithy.ipfs_client as ipfs_client

        ipfs_client.main(
            host=os.getenv("IPFS_ENDPOINT", "localhost"),
            action="upload",
            filePath="docker-compose-final.yml",
        )
        if not os.path.exists("IPFS_DOCKER_COMPOSE_HASH.ipfs"):
            print("Error: Could not upload docker-compose-final.yml to IPFS")
            sys.exit(1)
        with open("IPFS_DOCKER_COMPOSE_HASH.ipfs", "r") as f:
            IPFS_DOCKER_COMPOSE_HASH = f.read().strip()
        write_env("IPFS_DOCKER_COMPOSE_HASH", IPFS_DOCKER_COMPOSE_HASH)

        print("Upload docker registry to IPFS")
        ipfs_client.main(
            host=os.getenv("IPFS_ENDPOINT", "localhost"),
            action="upload",
            folderPath=registry_path,
        )
        if not os.path.exists("IPFS_HASH.ipfs"):
            print("Error: Could not upload docker registry to IPFS")
            sys.exit(1)
        with open("IPFS_HASH.ipfs", "r") as f:
            IPFS_HASH = f.read().strip()
        write_env("IPFS_HASH", IPFS_HASH)
        print("**** Finished ipfs initial pining ****")

        print("**** Started certificate generation ****")
        print("Requesting certificates from Ethernity Cloud...")
        print()

        import ethernity_cloud_sdk_py.commands.pynithy.run.public_key_service as public_key_service

        public_key_service.main(
            enclave_name=os.getenv("PROJECT_NAME", ""),
            protocol_version="v3",
            network=os.getenv("BLOCKCHAIN_NETWORK", ""),
            template_version=os.getenv("VERSION", ""),
        )

        if os.path.exists("PUBLIC_KEY.txt"):
            with open("PUBLIC_KEY.txt", "r") as f:
                PUBLIC_KEY_SECURELOCK_RES = f.read().strip()

        if (
            not PUBLIC_KEY_SECURELOCK_RES
            or "-----BEGIN CERTIFICATE-----" not in PUBLIC_KEY_SECURELOCK_RES
        ):
            print("Error: Could not fetch PUBLIC_KEY_SECURELOCK")
            sys.exit(1)

        # const CERTIFICATE_CONTENT_SECURELOCK = PUBLIC_KEY_SECURELOCK_RES.match(/-----BEGIN CERTIFICATE-----(.*?)-----END CERTIFICATE-----/s)[1].trim();
        # if (!CERTIFICATE_CONTENT_SECURELOCK) {
        #     console.error("ERROR! PUBLIC_KEY_SECURELOCK not found");
        #     process.exit(1);
        # } else {
        #     console.log("FOUND PUBLIC_KEY_SECURELOCK");
        # }
        # fs.writeFileSync('certificate.securelock.crt', PUBLIC_KEY_SECURELOCK_RES);
        # console.log("Listing certificate PUBLIC_KEY_SECURELOCK:");
        # console.log(fs.readFileSync('certificate.securelock.crt', 'utf8'))
        CERTIFICATE_CONTENT_SECURELOCK = (
            re.search(
                r"-----BEGIN CERTIFICATE-----(.*?)-----END CERTIFICATE-----",
                PUBLIC_KEY_SECURELOCK_RES,
                re.DOTALL,
            )
            .group(1)  # type: ignore
            .strip()
        )
        if not CERTIFICATE_CONTENT_SECURELOCK:
            print("Error: PUBLIC_KEY_SECURELOCK not found")
            sys.exit(1)
        CERTIFICATE_CONTENT_SECURELOCK = (
            "-----BEGIN CERTIFICATE-----\n"
            + CERTIFICATE_CONTENT_SECURELOCK
            + "\n-----END CERTIFICATE-----"
        )
        with open("certificate.securelock.crt", "w") as f:
            f.write(CERTIFICATE_CONTENT_SECURELOCK)
        print("Listing certificate PUBLIC_KEY_SECURELOCK:")
        print(CERTIFICATE_CONTENT_SECURELOCK)

        print("**** Finished certificate generation ****")

    print()
    print("getting PUBLIC_KEY_TRUSTEDZONE")
    if os.path.exists("certificate.trustedzone.crt"):
        os.remove("certificate.trustedzone.crt")

    import ethernity_cloud_sdk_py.commands.pynithy.run.image_registry as image_registry

    try:
        result = image_registry.main(
            os.getenv("BLOCKCHAIN_NETWORK", ""),
            templateName,
            "v3",
            "",
            "getTrustedZoneCert",
        )
    except Exception as e:
        print(e)
        result = ""

    trustedZoneCert = (
        re.search(
            r"-----BEGIN CERTIFICATE-----(.*?)-----END CERTIFICATE-----",
            result,  # type: ignore
            re.DOTALL,
        )
        .group(1)  # type: ignore
        .strip()
    )

    print("Listing certificate PUBLIC_KEY_TRUSTEDZONE:")
    print(trustedZoneCert)
    PUBLIC_KEY_TRUSTEDZONE = (
        "-----BEGIN CERTIFICATE-----\n"
        + trustedZoneCert
        + "\n-----END CERTIFICATE-----"
    )
    with open("certificate.trustedzone.crt", "w") as f:
        f.write(PUBLIC_KEY_TRUSTEDZONE)

    # copy both certificates to the registry folder
    shutil.copy("certificate.securelock.crt", registry_path)
    shutil.copy("certificate.trustedzone.crt", registry_path)

    # remove IPFS_HASH.ipfs files to upload them again with the certificates included
    if os.path.exists("IPFS_HASH.ipfs"):
        os.remove("IPFS_HASH.ipfs")

    print()
    print("**** Started ipfs pining ****")
    ipfs_client.main(
        host=os.getenv("IPFS_ENDPOINT", "localhost"),
        action="upload",
        folderPath=registry_path,
    )
    time.sleep(10)
    if not os.path.exists("IPFS_HASH.ipfs"):
        print("Error: Could not upload docker registry to IPFS")
        sys.exit(1)
    with open("IPFS_DOCKER_COMPOSE_HASH.ipfs", "r") as f:
        IPFS_DOCKER_COMPOSE_HASH = f.read().strip()
    write_env("IPFS_DOCKER_COMPOSE_HASH", IPFS_DOCKER_COMPOSE_HASH)
    os.environ["IPFS_DOCKER_COMPOSE_HASH"] = IPFS_DOCKER_COMPOSE_HASH

    with open("IPFS_HASH.ipfs", "r") as f:
        IPFS_HASH = f.read().strip()
    write_env("IPFS_HASH", IPFS_HASH)
    os.environ["IPFS_HASH"] = IPFS_HASH
    time.sleep(10)
    print("**** Finished ipfs pining ****")
    print()
    os.chdir(current_dir)
    print("Adding certificates for SECURELOCK into IMAGE REGISTRY smart contract...")
    print()
    load_dotenv()
    try:
        image_registry.main(
            os.getenv("BLOCKCHAIN_NETWORK", ""),
            os.getenv("PROJECT_NAME", ""),
            os.getenv("VERSION", ""),
            os.getenv("PRIVATE_KEY", ""),
            "registerSecureLockImage",
        )
    except Exception as e:
        print(e)
        exit()

    print()
    print("""
Your backend funcions were published successfully!
          
You can run the example cli application like this:

    python src/ethernity_task.py
        """
    )


if __name__ == "__main__":
    main()



import os.path
import ast

try:
    import serverless.backend as backend
except ImportError:
    backend = None
    pass

sdkFunctions = {}
if backend is not None:
    for func in backend.__dict__.keys():
        if func not in backend.__builtins__.keys() and func not in [
            "__file__",
            "__cached__",
            "__builtins__",
        ]:
            sdkFunctions.update({func: backend.__dict__[func]})


def ___etny_result___(data):
    quit([0, data])


class TaskStatus:
    SUCCESS = 0
    SYSTEM_ERROR = 1
    KEY_ERROR = 2
    SYNTAX_WARNING = 3
    BASE_EXCEPTION = 4
    PAYLOAD_NOT_DEFINED = 5
    PAYLOAD_CHECKSUM_ERROR = 6
    INPUT_CHECKSUM_ERROR = 7


def execute_task(payload_data, input_data):
    return Exec(
        payload_data,
        input_data,
        {"___etny_result___": ___etny_result___, **sdkFunctions},
    )


def Exec(payload_data, input_data, globals=None, locals=None):
    try:
        if payload_data is not None:
            if input_data is not None:
                globals["___etny_data_set___"] = input_data
            module = ast.parse(payload_data)
            outputs = []
            for node in module.body:
                if isinstance(node, ast.Expr):
                    expr_code = compile(
                        ast.Expression(node.value), filename="<ast>", mode="eval"
                    )
                    result = eval(expr_code, globals, locals)
                    outputs.append(result)
                else:
                    # Handle statements if needed
                    exec(
                        compile(ast.Module([node]), filename="<ast>", mode="exec"),
                        globals,
                        locals,
                    )

            return ___etny_result___("\n".join(outputs))
        else:
            return (
                TaskStatus.PAYLOAD_NOT_DEFINED,
                "Could not find the source file to execute",
            )

        return TaskStatus.SUCCESS, "TASK EXECUTED SUCCESSFULLY"
    except SystemError as e:
        return TaskStatus.SYSTEM_ERROR, e.args[0]
    except KeyError as e:
        return TaskStatus.KEY_ERROR, e.args[0]
    except SyntaxWarning as e:
        return TaskStatus.SYNTAX_WARNING, e.args[0]
    except BaseException as e:
        try:
            if e.args[0][0] == 0:
                return TaskStatus.SUCCESS, e.args[0][1]
            else:
                return TaskStatus.BASE_EXCEPTION, e.args[0]
        except Exception as e:
            return TaskStatus.BASE_EXCEPTION, e.args[0]


# payload_data = """hello('Iosif')
# hello('Luca')
# hello('World')"""
# print(execute_task(payload_data, None))
# print(execute_task(payload_data, None)[1])


# result = Exec('./v1/src/app/payload.py', './v1/src/app/input.txt',
#              {'etny_print': etny_print})
# print('task result:', result)

# write the task result in a file
# generate task hash



import os
import sys
import subprocess
from pathlib import Path
from dotenv import load_dotenv


def main():
    # Load environment variables from .env file
    current_dir = os.getcwd()
    dotenv_path = os.path.join(current_dir, ".env")
    print(f"Looking for .env file at: {dotenv_path}")
    load_dotenv(dotenv_path=dotenv_path)
    service_type = os.getenv("SERVICE_TYPE")
    print(f"Service type: {service_type}")
    if service_type == "Nodenithy":
        print("Adding prerequisites for Nodenithy...")
        script_path = Path(__file__).resolve().parent / "nodenithy" / "build.py"
        print(f"Running script: {script_path}")
        try:
            subprocess.run(["python", str(script_path)], check=True)
            print(
                "Build script finished. You can now proceed to publish: ecld-publish."
            )
        except subprocess.CalledProcessError:
            print("Error running the build script.")
            sys.exit(1)
    elif service_type == "Pynithy":
        print("Adding prerequisites for Pynithy...")
        import ethernity_cloud_sdk_py.commands.pynithy.build as buildScript

        # script_path = Path(__file__).resolve().parent / "pynithy" / "build.py"
        print(f"Running script: buildScript")
        # try:
        #     subprocess.run(["python", str(script_path)], check=True)
        #     print(
        #         "Build script finished. You can now proceed to publish: ecld-publish."
        #     )
        # except subprocess.CalledProcessError:
        #     print("Error running the build script.")
        #     sys.exit(1)
        try:
            buildScript.main()
            print(
                """
Build process was successful! You can now proceed to publish by running:

    ecld-publish
"""
            )
        except Exception as e:
            print(f"Error running the build script: {e}")
            sys.exit(1)
    else:
        print("Something went wrong")
        sys.exit(1)


if __name__ == "__main__":
    main()



import os
import sys
import shutil
import subprocess
import re
from pathlib import Path
from dotenv import load_dotenv, set_key

# For accessing package resources
try:
    from importlib.resources import path as resources_path
except ImportError:
    # For Python versions < 3.7
    from importlib_resources import path as resources_path  # type: ignore


def write_env(key, value, env_file=".env"):
    """
    Write or update key-value pairs in a .env file in the current working directory.
    """
    env_path = os.path.join(os.getcwd(), env_file)
    if not os.path.exists(env_path):
        with open(env_path, "w") as f:
            f.write(f"{key}={value}\n")
    else:
        replaced = False
        with open(env_path, "r") as f:
            lines = f.readlines()
        with open(env_path, "w") as f:
            for line in lines:
                if line.startswith(f"{key}="):
                    f.write(f"{key}={value}\n")
                    replaced = True
                else:
                    f.write(line)
            if not replaced:
                f.write(f"{key}={value}\n")


def get_project_name():
    """
    Prompt user for the project name.
    """
    while True:
        project_name = input("Choose a name for your project: ").strip()
        if not project_name:
            print("Project name cannot be blank. Please enter a valid name.")
        else:
            print(f"You have chosen the project name: {project_name}")
            return project_name


def display_options(options):
    for index, option in enumerate(options):
        print(f"{index + 1}) {option}")


def prompt_options(message, options, default_option):
    """
    Prompt the user to select an option.
    """
    while True:
        display_options(options)
        reply = input(message).strip()
        if not reply:
            print(f"No option selected. Defaulting to {default_option}.")
            return default_option
        elif reply.isdigit() and 1 <= int(reply) <= len(options):
            return options[int(reply) - 1]
        else:
            print(f"Invalid option {reply}. Please select a valid number.")


def print_intro():
    intro = """
    ╔───────────────────────────────────────────────────────────────────────────────────────────────────────────────╗
    │                                                                                                               │
    │        .... -+++++++. ....                                                                                    │
    │     -++++++++-     .++++++++.      _____ _   _                     _ _             ____ _                 _   │
    │   .++-     ..    .++-     .++-    | ____| |_| |__   ___ _ __ _ __ (_) |_ _   _    / ___| | ___  _   _  __| |  │
    │  --++----      .++-         ...   |  _| | __| '_ \\ / _ \\ '__| '_ \\| | __| | | |  | |   | |/ _ \\| | | |/ _` |  │
    │  --++----    .++-.          ...   | |___| |_| | | |  __/ |  | | | | | |_| |_| |  | |___| | (_) | |_| | (_| |  │
    │   .++-     .+++.    .     .--.    |_____|\\__|_| |_|\\___|_|  |_| |_|_|\\__|\\__, |   \\____|_|\\___/ \\__,_|\\__,_|  │
    │     -++++++++.    .---------.                                            |___/                                │
    │        .... .-------. ....                                                                                    │
    │                                                                                                               │
    ╚───────────────────────────────────────────────────────────────────────────────────────────────────────────────╝
                                          Welcome to the Ethernity Cloud SDK

       The Ethernity Cloud SDK is a comprehensive toolkit designed to facilitate the development and management of
      decentralized applications (dApps) and serverless binaries on the Ethernity Cloud ecosystem. Geared towards
      developers proficient in Python or Node.js, this toolkit aims to help you effectively harness the key features
      of the ecosystem, such as data security, decentralized processing, and blockchain-driven transparency and
      trustless model for real-time data processing.
    """
    print(intro)


def main():
    print_intro()
    project_name = get_project_name()
    print()
    service_type_options = ["Pynithy", "Custom"]  # "Nodenithy",
    service_type = prompt_options(
        "Select the type of code to be ran during the compute layer (default is Pynithy): ",
        service_type_options,
        "Pynithy",
    )

    docker_repo_url = docker_login = docker_password = base_image_tag = None
    if service_type == "Custom":
        docker_repo_url = input("Enter Docker repository URL: ").strip()
        docker_login = input("Enter Docker Login (username): ").strip()
        docker_password = input("Enter Password: ").strip()
        base_image_tag = input("Enter the image tag: ").strip()
    print()
    blockchain_network_options = [
        "Bloxberg Mainnet",
        "Bloxberg Testnet",
        "Polygon Mainnet",
        "Polygon Amoy Testnet",
    ]
    blockchain_network = prompt_options(
        "On which Blockchain network do you want to have the app set up, as a starting point? (default is Bloxberg Testnet): ",
        blockchain_network_options,
        "Bloxberg Testnet",
    )
    print()

    print(
        f"Checking if the project name (image name) is available on the {blockchain_network.replace(' ', '_')} network and ownership..."
    )
    import ethernity_cloud_sdk_py.commands.pynithy.run.image_registry as image_registry

    # Execute the external script (image_registry.py)
    # script_path = (
    #     Path(__file__).resolve().parent / "pynithy" / "run" / "image_registry.py"
    # )
    print(f"Running script image_registry...")
    print(os.getcwd())
    # if not script_path.exists():
    #     print(f"Error: Script {script_path} not found.")
    #     sys.exit(1)

    # try:
    #     subprocess.run(
    #         [
    #             "python",
    #             str(script_path),
    #             blockchain_network.replace(" ", "_"),
    #             project_name.replace(" ", "-"),
    #             "v3",
    #         ],
    #         check=True,
    #     )
    # except subprocess.CalledProcessError as e:
    #     print(f"Error executing script {script_path}")
    #     sys.exit(1)
    image_registry.main(
        blockchain_network.replace(" ", "_"),
        project_name.replace(" ", "-"),
        "v3",
    )

    print()
    ipfs_service_options = ["Ethernity (best effort)", "Custom IPFS"]
    ipfs_service = prompt_options(
        "Select the IPFS pinning service you want to use (default is Ethernity): ",
        ipfs_service_options,
        "Ethernity (best effort)",
    )

    custom_url = ipfs_token = None
    if ipfs_service == "Custom IPFS":
        custom_url = input(
            "Enter the endpoint URL for the IPFS pinning service you want to use: "
        ).strip()
        ipfs_token = input(
            "Enter the access token to be used when calling the IPFS pinning service: "
        ).strip()
    else:
        custom_url = "https://ipfs.ethernity.cloud"

    os.makedirs("src/serverless", exist_ok=True)

    print()
    app_template_options = ["yes", "no"]
    use_app_template = prompt_options(
        "Do you want a 'Hello World' app template as a starting point? (default is yes): ",
        app_template_options,
        "yes",
    )

    if use_app_template == "yes":
        print("Bringing Cli/Backend templates...")
        print("  src/serverless/backend.py (Hello World function)")
        print("  src/ethernity_task.py (Hello World function call - Cli)")
        # Copy the 'src' and 'public' directories from the package to the current directory
        # We need to use package resources for this
        package_name = "ethernity_cloud_sdk_py"
        # Copy 'src' directory
        with resources_path(f"{package_name}.templates", "src") as src_path:
            shutil.copytree(
                src_path, os.path.join(os.getcwd(), "src"), dirs_exist_ok=True
            )
        # Simulate copying files
        # script_dir = os.path.dirname(os.path.abspath(__file__))

        # source_src = os.path.join(script_dir, "src")
        # target_src = "src/"
        # # source_public = os.path.join(script_dir, "public")
        # # target_public = "public/"

        # shutil.copytree(source_src, target_src, dirs_exist_ok=True)
        # shutil.copytree(source_public, target_public, dirs_exist_ok=True)
        # print("Installing required packages...")
        # # Simulate npm install
        # try:
        #     subprocess.run(
        #         [
        #             "npm",
        #             "install",
        #             "@ethernity-cloud/runner@0.0.26",
        #             "@testing-library/jest-dom@5.17.0",
        #             "@testing-library/react@13.4.0",
        #             "@testing-library/user-event@13.5.0",
        #             "react@18.3.1",
        #             "react-dom@18.3.1",
        #             "react-scripts@5.0.1",
        #             "web-vitals@2.1.4",
        #             "web3@4.9.0",
        #             "dotenv@16.4.5",
        #         ],
        #         check=True,
        #     )
        # except subprocess.CalledProcessError as e:
        #     print("Error installing npm packages.")
        #     sys.exit(1)
    else:
        print(
            "Define backend functions in src/serverless to be available for cli interaction."
        )

    write_env("PROJECT_NAME", project_name.replace(" ", "_"))
    write_env("SERVICE_TYPE", service_type)
    if service_type == "Custom":
        write_env("BASE_IMAGE_TAG", base_image_tag or "")
        write_env("DOCKER_REPO_URL", docker_repo_url)
        write_env("DOCKER_LOGIN", docker_login)
        write_env("DOCKER_PASSWORD", docker_password)
    elif service_type == "Nodenithy":
        write_env("BASE_IMAGE_TAG", "")
        write_env("DOCKER_REPO_URL", "")
        write_env("DOCKER_LOGIN", "")
        write_env("DOCKER_PASSWORD", "")
    elif service_type == "Pynithy":
        write_env("BASE_IMAGE_TAG", "")
        write_env("DOCKER_REPO_URL", "")
        write_env("DOCKER_LOGIN", "")
        write_env("DOCKER_PASSWORD", "")
    write_env("BLOCKCHAIN_NETWORK", blockchain_network.replace(" ", "_"))
    write_env("IPFS_ENDPOINT", custom_url)
    write_env("IPFS_TOKEN", ipfs_token or "")
    write_env("VERSION", "v1")

    write_env(
        "TRUSTED_ZONE_IMAGE",
        f"{'ecld' if 'polygon' in blockchain_network.lower() else 'etny'}-{service_type.lower()}-{'testnet' if 'testnet' in blockchain_network.lower() else ''}",
    )

    print()
    print(
        """=================================================================================================================

The customize the backend edit serverless/backend.py with your desired functions.
Please skip this step if you only want to run the helloworld example.

Now you are ready to build!
To start the build process run:

    ecld-build
        """
    )


if __name__ == "__main__":
    main()



import requests
import time
import argparse

BASE_URL = "https://publickey.ethernity.cloud"


def submit_ipfs_hash(
    hhash,
    enclave_name,
    protocol_version,
    network,
    template_version,
    docker_composer_hash,
):
    url = f"{BASE_URL}/api/addHash"
    payload = {
        "hash": hhash,
        "enclave_name": enclave_name,
        "protocol_version": protocol_version,
        "network": network,
        "template_version": template_version,
        "docker_composer_hash": docker_composer_hash,
    }
    response = requests.post(url, json=payload)
    print(f"response:{response}")
    if response.status_code == 200:
        return response.json()
    else:
        print("Error:", response.json())
        exit(1)


def check_ipfs_hash_status(hash):
    url = f"{BASE_URL}/api/checkHash/{hash}"
    response = requests.get(url)
    if response.status_code == 200:
        return response.json()
    else:
        print("Error:", response.json())
        exit(1)


def main(
    enclave_name,
    protocol_version,
    network,
    template_version,
    hhash="",
    docker_composer_hash="",
):
    print("Enclave Name:", enclave_name)
    print("Protocol Version:", protocol_version)
    print("Network:", network)
    print("Template Version:", template_version)

    if not hhash:
        with open("IPFS_HASH.ipfs", "r") as f:
            hhash = f.read().strip()
        with open("IPFS_DOCKER_COMPOSE_HASH.ipfs", "r") as f:
            docker_composer_hash = f.read().strip()

    print()
    print("IPFS Hash:", hhash)
    print("Docker Composer Hash:", docker_composer_hash)
    print()

    # Submit IPFS Hash
    submit_response = submit_ipfs_hash(
        hhash,
        enclave_name,
        protocol_version,
        network,
        template_version,
        docker_composer_hash,
    )
    print("Submit IPFS Hash Response:", submit_response)

    # Check IPFS Hash Status
    while True:
        check_response = check_ipfs_hash_status(hhash)
        if "publicKey" in check_response:
            if check_response["publicKey"] == 0:
                print(
                    f"Public key not available yet. Queue position: {check_response.get('queuePosition', 'Unknown')}"
                )
            elif check_response["publicKey"] == -1:
                print("Hash is not derived from Eternity Cloud SDK.")
                exit(1)
            else:
                print("Public Key:", check_response["publicKey"])
                # save public key to file
                with open("PUBLIC_KEY.txt", "w") as f:
                    f.write(check_response["publicKey"])
                break
        else:
            print("Unexpected response:", check_response)
            exit(1)

        time.sleep(10)  # Wait for 10 seconds before checking again


if __name__ == "__main__":
    # Example usage
    # python3 ./public_key_service.py --enclave_name ${ENCLAVE_NAME_SECURELOCK} --protocol_version ${VERSION} --network ${BLOCKCHAIN_NETWORK} --template_version ${VERSION}

    # read enclave_name, protocol_version, network, template_version from command line arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("--enclave_name", help="Enclave name", required=True)
    parser.add_argument("--protocol_version", help="Protocol version", required=True)
    parser.add_argument("--network", help="Network", required=True)
    parser.add_argument("--template_version", help="Template version", required=True)

    args = parser.parse_args()
    enclave_name = args.enclave_name
    protocol_version = args.protocol_version
    network = args.network.lower().split("_")[1]
    template_version = args.template_version

    hhash = ""
    with open("IPFS_HASH.ipfs", "r") as f:
        hhash = f.read().strip()
    docker_composer_hash = ""
    with open("IPFS_DOCKER_COMPOSE_HASH.ipfs", "r") as f:
        docker_composer_hash = f.read().strip()

    main(
        enclave_name=enclave_name,
        protocol_version=protocol_version,
        network=network,
        template_version=template_version,
        hhash=hhash,
        docker_composer_hash=docker_composer_hash,
    )



import hashlib
from eth_account import Account

def get_wallet_address(first_string, second_string):
    concat = (first_string + second_string).encode()
    _hash1 = hashlib.sha256(concat).hexdigest()
    _hash = hashlib.sha256(_hash1.encode()).hexdigest()
    acct = Account.from_key(_hash)
    return acct.address, _hash


if __name__ == '__main__':
    first_string = 'test'
    second_string = 'test'
    (addr, private) = get_wallet_address(first_string, second_string)
    print('account address = ', addr)  # 0xf63106856F7007A30025f0fFD5A534EA880878C3
    print('private = ', private)



from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
setup(
    name="ethernity-cloud-sdk-py",
    version="0.1.39",
    url="https://github.com/ethernity-cloud/ethernity-cloud-sdk-py",
    author="Ethernity Cloud Team",
    author_email="contact@ethernity.cloud",
    description="Ethernity Cloud SDK Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    package_data={
        "ethernity_cloud_sdk_py.templates": ["src/**/*"],
    },
    install_requires=[
        "requests",
        "python-dotenv",
        "tqdm",
        "pyopenssl",
        "requests_toolbelt",
        "cryptography",
        "typing-extensions",
        "ethernity-cloud-runner-py",
        # Add other dependencies here
    ],
    entry_points={
        "console_scripts": [
            "ecld-init=ethernity_cloud_sdk_py.cli:main_init",
            "ecld-build=ethernity_cloud_sdk_py.cli:main_build",
            "ecld-publish=ethernity_cloud_sdk_py.cli:main_publish",
        ],
    },
)



from __future__ import annotations

import base64
import binascii
import codecs
import hashlib
import json
import secrets
from pyasn1.codec.der import decoder
from Crypto.Cipher import AES
from tinyec import registry
from tinyec import ec
from Crypto.PublicKey import ECC
from nacl.public import Box, PrivateKey, PublicKey
from base64 import a85encode


class etny_crypto:
    curve = registry.get_curve('secp384r1')

    @staticmethod
    def compress_point(point):
        return hex(point.x) + hex(point.y % 2)[2:]

    @staticmethod
    def ecc_point_to_256_bit_key(point):
        value = str(point.x) + str(point.y)
        sha = hashlib.sha256(value.encode())
        return sha.digest()

    @staticmethod
    def encrypt_aes_gcm(msg, secret_key):
        aes_cipher = AES.new(secret_key, AES.MODE_GCM)
        cipher_text, auth_tag = aes_cipher.encrypt_and_digest(msg)
        return (cipher_text, aes_cipher.nonce, auth_tag)

    @staticmethod
    def encrypt_ecc(msg, public_key):
        cipher_text_private_key = secrets.randbelow(etny_crypto.curve.field.n)
        shared_ecc_key = cipher_text_private_key * public_key
        secret_key = etny_crypto.ecc_point_to_256_bit_key(shared_ecc_key)
        ciphertext, nonce, auth_tag = etny_crypto.encrypt_aes_gcm(msg, secret_key)
        cipher_text_public_key = cipher_text_private_key * etny_crypto.curve.g
        return (ciphertext, nonce, auth_tag, cipher_text_public_key)

    @staticmethod
    def decrypt_ase_gcm(ciphertext, nonce, auth_tag, secret_key):
        aes_cipher = AES.new(secret_key, AES.MODE_GCM, nonce)
        plaintext = aes_cipher.decrypt_and_verify(ciphertext, auth_tag)
        return plaintext

    @staticmethod
    def decrypt_ecc(encrypted_msg, priv_key):
        (ciphertext, nonce, authTag, ciphertextPubKey) = encrypted_msg
        shared_ecc_key = priv_key * ciphertextPubKey
        secret_key = etny_crypto.ecc_point_to_256_bit_key(shared_ecc_key)
        plaintext = etny_crypto.decrypt_ase_gcm(ciphertext, nonce, authTag, secret_key)
        return plaintext

    @staticmethod
    def ecc_read_key(public_key_file):
        with open(public_key_file) as r:
            key = ECC.import_key(r.read())
            return key

    @staticmethod
    def encrypt(public_key_file, message):
        with open(public_key_file) as f:
            ecc_public_cert = ECC.import_key(f.read())
            pub_key_ecc_point = ec.Point(etny_crypto.curve, ecc_public_cert.pointQ.x.__int__(),
                                         ecc_public_cert.pointQ.y.__int__())
            encrypted_msg = etny_crypto.encrypt_ecc(message, pub_key_ecc_point)
            return encrypted_msg

    @staticmethod
    def encrypt_with_pub_key(pub_key_content, message):
        ecc_public_cert = ECC.import_key(pub_key_content)
        pub_key_ecc_point = ec.Point(etny_crypto.curve, ecc_public_cert.pointQ.x.__int__(),
                                     ecc_public_cert.pointQ.y.__int__())
        encrypted_msg = etny_crypto.encrypt_ecc(message, pub_key_ecc_point)
        return encrypted_msg

    @staticmethod
    def clean_private_key(private_key_data):
        private_key_data = private_key_data.replace(b"-----BEGIN PRIVATE KEY-----", b"")
        private_key_data = private_key_data.replace(b"-----END PRIVATE KEY-----", b"")
        private_key_data = codecs.decode(private_key_data, 'base64')
        return private_key_data

    @staticmethod
    def decrypt(private_key_file, encrypted_msg):
        # Reading and calculating Private Key from PEM
        with open(private_key_file) as f:
            private_key_data = str.encode(f.read())

        private_key_data = etny_crypto.clean_private_key(private_key_data)
        # decode der with asn1 library
        # - get the octet string (field-2) containing the raw key
        asn1_object, _ = decoder.decode(private_key_data)
        raw_keys = asn1_object.getComponentByName('field-2').asOctets()
        # - get the octet string (field-1) containing the raw private key
        #   and the bit string (field-2) containing the uncompressed public key
        asn1_object, _ = decoder.decode(raw_keys)
        private_key = asn1_object.getComponentByName('field-1').asOctets()

        # Generating keypair for tinyEC
        priv_key_tec = int.from_bytes(private_key, byteorder="big")
        decrypted_msg = etny_crypto.decrypt_ecc(encrypted_msg, priv_key_tec)
        #print("\ndecrypted msg:", decrypted_msg)
        return decrypted_msg

    @staticmethod
    def encrypted_data_to_base64_json(encrypted_msg):
        json_obj = {
            'ciphertext': binascii.hexlify(encrypted_msg[0]).decode(),
            'nonce': binascii.hexlify(encrypted_msg[1]).decode(),
            'authTag': binascii.hexlify(encrypted_msg[2]).decode(),
            'x': hex(encrypted_msg[3].x),
            'y': hex(encrypted_msg[3].y)
        }

        json_string = json.dumps(json_obj)
        binary_data = json_string.encode('utf-8')
        base64_data = base64.b64encode(binary_data)

        return base64_data

    @staticmethod
    def encrypted_data_from_base64_json(base64_data):
        # decode the base64-encoded string back to a JSON string
        decoded_json_data = base64.b64decode(base64_data).decode('utf-8')

        json_load = json.loads(decoded_json_data)
        return (binascii.unhexlify(json_load['ciphertext'].encode()),
                binascii.unhexlify(json_load['nonce'].encode()),
                binascii.unhexlify(json_load['authTag'].encode()),
                ec.Point(etny_crypto.curve, int(json_load['x'], 16),
                         int(json_load['y'], 16)))

    @staticmethod
    def hex_to_bytes(hex_str: str) -> bytes:
        return bytes.fromhex(hex_str[2:] if hex_str[:2] == "0x" else hex_str)

    @staticmethod
    def encrypt_nacl(public_key: str, data: bytes) -> str:
        """Encryption function using NaCl box compatible with MetaMask
        For implementation used in MetaMask look into: https://github.com/MetaMask/eth-sig-util
        Args:
            public_key: public key of recipient (32 bytes)
            data: message data
        Returns:
            encrypted data
        """
        emph_key = PrivateKey.generate()
        enc_box = Box(emph_key, PublicKey(etny_crypto.hex_to_bytes(public_key)))
        # Encryption must work with MetaMask decryption (requires valid utf-8)
        data = a85encode(data)
        ciphertext = enc_box.encrypt(data)
        result = bytes(emph_key.public_key) + ciphertext
        return result.hex()


if __name__ == '__main__':
    encrypted_msg_ = etny_crypto.encrypt('./app/cert1-ca1-clean.crt', b'test')
    #print(encrypted_msg_)
    # etny_crypto.decrypt('./certs/cert1-ca1-clean.key', encrypted_msg_)
    #print(etny_crypto.encrypted_data_to_base64_json(encrypted_msg_))



import os
import sys
import warnings
import time
from datetime import datetime
from dotenv import load_dotenv

warnings.filterwarnings("ignore")

try:
    load_dotenv(".env" if os.path.exists(".env") else ".env.config")
except ImportError as e:
    pass

from ethernity_cloud_runner_py.runner import EthernityCloudRunner

code = "hello('World!')"

def execute_task(code) -> None:
    runner = EthernityCloudRunner()
    runner.set_log_level("DEBUG")

    runner.set_private_key(os.environ.get("PRIVATE_KEY"))
    runner.set_network("Bloxberg", "Testnet")
    runner.set_storage_ipfs("https://ipfs.ethernity.cloud/api/v0")

    runner.connect()

  
    resources = {
        "taskPrice": 3,
        "cpu": 4,
        "memory": 3,
        "storage": 10,
        "bandwidth": 1,
        "duration": 1,
        "validators": 1,
    }

    trustedzone_enclave = os.getenv("TRUSTED_ZONE_IMAGE")
    securelock_enclave = os.getenv("PROJECT_NAME")
    
    runner.run(
        resources,
        securelock_enclave,
        code,
        "",
        trustedzone_enclave
    )

    # Store previously printed logs
    previous_logs = set()

    while runner.is_running():
        state = runner.get_state()
        current_logs = set(state['log'])  # Current set of logs

        # Find new logs by subtracting previous_logs from current_logs
        new_logs = current_logs - previous_logs

        # If there are new logs, print them
        if new_logs:
            for log in new_logs:
                print(log)
        
        # Update previous_logs to include the current logs
        previous_logs = current_logs

        # Optional status prints
        # print(f"{datetime.now()} Task status: {state['progress']}")
        # print(f"Processed Events: {state['processed_events']}, Remaining Events: {state['remaining_events']}")    

    time.sleep(0.5)
    state = runner.get_state()

    if state['status'] == "ERROR":
        for log in state['log']:
            print(log)
        print(f"Processed Events: {state['processed_events']}, Remaining Events: {state['remaining_events']}")
        
    elif state['status'] == "SUCCESS":    
        result = runner.get_result()
        print(result['value'])

if __name__ == "__main__":
    execute_task(code)
